﻿using SkiaSharp;

namespace AiYoloV5Onnx.Common
{
    public static class BitmapExt
    {
        /// <summary>
        /// 裁剪大小
        /// </summary>
        /// <param name="source">SKBitmap</param>
        /// <param name="new_width">新的宽度</param>
        /// <param name="new_height">新的高度</param>
        /// <returns></returns>
        public static SKBitmap Resize(this SKBitmap source, int new_width, int new_height)
        {
            //等比例缩小
            var newScaleDown = NewScaleDown(source.Width, source.Height, new_width, new_height);
            var newWidth = newScaleDown.Item1;
            var newHeight = newScaleDown.Item2;
            var newLeft = newScaleDown.Item3;
            var newTop = newScaleDown.Item4;
            //缩放
            //var skimageinfo = new SKImageInfo(newWidth, newHeight, SKColorType.Rgb888x, SKAlphaType.Opaque);
            //var skSizeI = new SKSizeI(newWidth, newHeight);
            //var resized = source.Resize(skSizeI, SKFilterQuality.Medium);
            //目标画板
            var image2d = new SKBitmap(new_width, new_height);
            //画布
            using var canvas = new SKCanvas(image2d);
            //填充背景
            //canvas.DrawColor(SKColors.Black);
            //绘制到画板
            var skRect = new SKRect(newLeft, newTop, new_width - newLeft, new_height - newTop);
            canvas.DrawBitmap(source, skRect);
            ////保存图片
            //using (var writeStream = new FileStream(@"D:\123.jpg", FileMode.Create))
            //{
            //    using var image = SKImage.FromBitmap(image2d);
            //    image.Encode().SaveTo(writeStream);
            //}
            //
            return image2d;
        }

        /// <summary>
        /// 绘制信息
        /// </summary>
        public static void DrawYoloPrediction(this SKBitmap source, List<Prediction> predictions, SKColor boxColor, SKColor labelColor, int new_width, int new_height)
        {
            if (predictions is null || predictions.Count == 0) return;
            //等比例缩小
            var newScaleDown = NewScaleDown(source.Width, source.Height, new_width, new_height);
            var newWidth = newScaleDown.Item1;
            var newHeight = newScaleDown.Item2;
            var newLeft = newScaleDown.Item3;
            var newTop = newScaleDown.Item4;
            //画布
            using var canvas = new SKCanvas(source);
            //
            foreach (var p in predictions)
            {
                var top = (int)((p.Box.MinY * new_height - newTop) / newHeight * source.Height);
                var left = (int)((p.Box.MinX * new_width - newLeft) / newWidth * source.Width);
                var bottom = (int)((p.Box.MaxY * new_height - newTop) / newHeight * source.Height);
                var right = (int)((p.Box.MaxX * new_width - newLeft) / newWidth * source.Width);
                //可阅读的边界信息
                p.Points = new()
                {
                    top = top,
                    left = left,
                    bottom = bottom,
                    right = right,
                };
                //方形样式
                using var drawStyleA = new SKPaint();
                drawStyleA.Color = boxColor;
                drawStyleA.StrokeWidth = 1;
                drawStyleA.Style = SKPaintStyle.Stroke;//只绘制边框
                canvas.DrawRect(left, top, right - left, bottom - top, drawStyleA);
                //字体样式
                var textSize = 22f;
                var textFont = SKTypeface.FromFamilyName("宋体", SKFontStyleWeight.SemiBold, SKFontStyleWidth.ExtraCondensed, SKFontStyleSlant.Upright);
                using var drawStyleB = new SKPaint();
                drawStyleB.Color = labelColor;
                drawStyleB.IsAntialias = true;
                drawStyleB.TextSize = textSize;
                drawStyleB.Typeface = textFont;
                canvas.DrawText($"{p.LabelName}, {p.Confidence:0.00}", left, top + textSize, drawStyleB);
            }
        }

        /// <summary>
        /// 等比例缩放图像
        /// </summary>
        internal static (int, int, int, int) NewScaleDown(int width, int height, int new_width, int new_height)
        {
            //等比例缩小
            var newWidth = new_width;
            var newHeight = (height * newWidth) / width;
            var newLeft = 0;
            var newTop = (new_height - newHeight) / 2;
            if (width <= height)
            {
                newWidth = (width * newWidth) / height;
                newHeight = new_height;
                newLeft = (new_width - newWidth) / 2;
                newTop = 0;
            }
            return (newWidth, newHeight, newLeft, newTop);
        }
    }
}