﻿using System.Diagnostics;

namespace AiYoloV5Onnx.Common
{
    public class Prediction
    {
        /// <summary>
        /// 边框信息（原始）
        /// </summary>
        internal BBox Box { get; set; }

        /// <summary>
        /// 边框信息
        /// </summary>
        public Location Points { get; set; }

        /// <summary>
        /// 标签索引
        /// </summary>
        public int LabelIndex { get; set; }

        /// <summary>
        /// 标签名称
        /// </summary>
        public string? LabelName { get; set; }

        /// <summary>
        /// 自信度
        /// </summary>
        public float Confidence { get; set; }

        /// <summary>
        /// 去除重复和重叠的边界框
        /// </summary>
        public static List<Prediction> NMS(List<Prediction> predictions, float IOU_threshold = 0.45f)
        {
            var final_predications = new List<Prediction>();

            for (int i = 0; i < predictions.Count; i++)
            {
                int j = 0;
                for (j = 0; j < final_predications.Count; j++)
                {
                    if (ComputeIOU(final_predications[j], predictions[i]) > IOU_threshold)
                    {
                        break;
                    }
                }
                if (j == final_predications.Count)
                {
                    final_predications.Add(predictions[i]);
                }
            }
            return final_predications;
        }

        private static float ComputeIOU(Prediction DRa, Prediction DRb)
        {
            float ay1 = DRa.Box.MinY;
            float ax1 = DRa.Box.MinX;
            float ay2 = DRa.Box.MaxY;
            float ax2 = DRa.Box.MaxX;
            float by1 = DRb.Box.MinY;
            float bx1 = DRb.Box.MinX;
            float by2 = DRb.Box.MaxY;
            float bx2 = DRb.Box.MaxX;

            Debug.Assert(ay1 < ay2);
            Debug.Assert(ax1 < ax2);
            Debug.Assert(by1 < by2);
            Debug.Assert(bx1 < bx2);

            // determine the coordinates of the intersection rectangle
            float x_left = Math.Max(ax1, bx1);
            float y_top = Math.Max(ay1, by1);
            float x_right = Math.Min(ax2, bx2);
            float y_bottom = Math.Min(ay2, by2);

            if (x_right < x_left || y_bottom < y_top) return 0;
            //
            float intersection_area = (x_right - x_left) * (y_bottom - y_top);
            float bb1_area = (ax2 - ax1) * (ay2 - ay1);
            float bb2_area = (bx2 - bx1) * (by2 - by1);
            float iou = intersection_area / (bb1_area + bb2_area - intersection_area);

            Debug.Assert(iou >= 0 && iou <= 1);
            return iou;
        }
    }

    /// <summary>
    /// 边框信息
    /// </summary>
    internal record BBox(float MinX, float MinY, float MaxX, float MaxY);

    /// <summary>
    /// 位置信息
    /// </summary>
    public struct Location
    {
        public int top { get; set; }
        public int left { get; set; }
        public int bottom { get; set; }
        public int right { get; set; }
    }
}