﻿using AiYoloV5Onnx.Yolo;
using Microsoft.AspNetCore.Mvc;
using SkiaSharp;

namespace AiYoloV5Onnx.Controllers
{
    public class HomeController : Controller
    {
        private IWebHostEnvironment _WebHost { get; }
        private Yolov5Detector _Yolov5Detector { get; }

        public HomeController(IWebHostEnvironment webHost, Yolov5Detector yolov5Detector)
        {
            this._WebHost = webHost;
            this._Yolov5Detector = yolov5Detector;
        }

        public IActionResult Index()
        {
            return View();
        }

        public IActionResult Update(IFormFile file)
        {
            if (!file.ContentType.Contains("image")) return Json("Error");
            //AI识别
            using var fileStream = new SKManagedStream(file.OpenReadStream());
            var bitmap = SKBitmap.Decode(fileStream);
            var preds = _Yolov5Detector.Inference(bitmap);
            //创建对象信息
            using var img = SKImage.FromBitmap(bitmap);
            using var sKData = img.Encode();
            //保存到流
            using var ms = new MemoryStream();
            sKData.SaveTo(ms);
            var bytes = ms.GetBuffer();
            ////创建对象信息
            //using var img = SKImage.FromBitmap(bitmap);
            //using var p = img.Encode(SKEncodedImageFormat.Png, 100);
            ////保存到流
            //using var ms = new MemoryStream();
            //p.SaveTo(ms);
            //var bytes = ms.GetBuffer();
            //返回
            return File(bytes, "image/jpg");
        }

        public IActionResult UpdateInfo(IFormFile file)
        {
            if (!file.ContentType.Contains("image")) return Json("Error");
            //AI识别
            using var fileStream = new SKManagedStream(file.OpenReadStream());
            var bitmap = SKBitmap.Decode(fileStream);
            var preds = _Yolov5Detector.Inference(bitmap);
            //返回
            return Json(preds);
        }

        public IActionResult UpdateFile(IFormFile file)
        {
            if (!file.ContentType.Contains("image")) return Json("Error");
            //AI识别
            using var fileStream = new SKManagedStream(file.OpenReadStream());
            var bitmap = SKBitmap.Decode(fileStream);
            var preds = _Yolov5Detector.Inference(bitmap);
            //保存文件（上传的文件）
            var filePath = $"{_WebHost.WebRootPath}/UploadFile/";
            if (!Directory.Exists(filePath)) Directory.CreateDirectory(filePath);
            filePath += file.FileName;
            using (var inputStream = new FileStream(filePath, FileMode.Create))
            {
                file.CopyTo(inputStream);
                var array = new byte[inputStream.Length];
                inputStream.Seek(0, SeekOrigin.Begin);
                inputStream.Read(array, 0, array.Length);
            }
            //保存文件（识别后的文件）
            var fileAiPath = $"{_WebHost.WebRootPath}/UploadFile/Ai/";
            if (!Directory.Exists(fileAiPath)) Directory.CreateDirectory(fileAiPath);
            fileAiPath += file.FileName;
            using (var writeStream = new FileStream(fileAiPath, FileMode.Create))
            {
                using var image = SKImage.FromBitmap(bitmap);
                image.Encode(SKEncodedImageFormat.Png, 80).SaveTo(writeStream);
            }
            //返回
            return View();
        }
    }
}