using AiYoloV5Onnx.Yolo;
using System.Text;
using System.Text.Encodings.Web;
using System.Text.Unicode;

namespace AiYoloV5Onnx
{
    public static class Program
    {
        public static void Main(string[] args)
        {
            var builder = WebApplication.CreateBuilder(args);
            builder.Services.ConfigureServices(builder);
            ////限制请求大小
            //builder.WebHost.UseKestrel(options =>
            //{
            //    options.Limits.MaxRequestLineSize = 2000000;//HTTP 请求行的最大允许大小，默认为 8kb
            //    options.Limits.MaxRequestBufferSize = 2000000;//请求缓冲区的最大大小，默认为 1M
            //    options.Limits.MaxRequestBodySize = 2000000;//限制请求长度，默认 30,000,000 字节
            //});
            ////限制上传大小
            //builder.Services.Configure<FormOptions>(x =>
            //{
            //    x.ValueLengthLimit = 2000000;
            //    x.MultipartBodyLengthLimit = 2000000;
            //});
            var app = builder.Build();
            app.Configure(builder);
            app.Run();
        }

        /// <summary>
        /// 该方法通过运行时调用
        /// 使用此方法将服务添加到容器中
        /// </summary>
        /// <param name="services">服务</param>
        /// <param name="builder">网站程序</param>
        public static void ConfigureServices(this IServiceCollection services, WebApplicationBuilder builder)
        {
            //添加单例
            services.AddSingleton(HtmlEncoder.Create(UnicodeRanges.All));
            //注册编码
            Encoding.RegisterProvider(CodePagesEncodingProvider.Instance);
            //返回数据首字母不小写
            services.AddMvc().AddJsonOptions(options =>
            {
                //options.JsonSerializerOptions.PropertyNamingPolicy = null;
            });
            //启用缓存功能
            services.AddMemoryCache();
            //启动 Session
            services.AddSession(options =>
            {
                options.Cookie.Name = ".AspNetCore.Session";
                options.IdleTimeout = TimeSpan.FromDays(7);//设置Session的过期时间
                options.Cookie.HttpOnly = true;//设置在浏览器不能通过js获得该Cookie的值
                options.Cookie.IsEssential = true;
            });
            //
            //services.AddScoped(x => new Yolov5Detector($"{builder.Environment.ContentRootPath}/yolov5m6.onnx", $"{builder.Environment.ContentRootPath}/yolov5m.txt", 1280, 1280));
            services.AddScoped(x => new Yolov5Detector($"{builder.Environment.ContentRootPath}/yolov5m.onnx", $"{builder.Environment.ContentRootPath}/yolov5m.txt"));
            //添加 Options 模式
            services.AddOptions();
            //添加 MVC
            services.AddMvc();
            //添加 HttpContext 存取器
            services.AddHttpContextAccessor();
        }

        /// <summary>
        /// 该方法通过运行时调用
        /// 使用此方法配置HTTP请求流水线
        /// </summary>
        /// <param name="app">应用</param>
        /// <param name="builder">网站程序</param>
        public static void Configure(this WebApplication app, WebApplicationBuilder builder)
        {
            //默认的静态目录路径
            app.UseStaticFiles();
            //用户 Session
            app.UseSession();
            //用户路由
            app.UseRouting();
            //用户授权
            app.UseAuthorization();
            //用户默认路由
            app.MapControllerRoute(
                name: "areaRoute",
                pattern: "{area:exists}/{controller=Home}/{action=Index}/{id?}");
            app.MapControllerRoute(
                name: "default",
                pattern: "{controller=Home}/{action=Index}/{id?}");
        }
    }
}